/* scanner for a toy Pascal-like language */

%option noyywrap
%{
#include <stdlib.h>
#include <malloc.h>
#include <math.h> /* need this for the call to atof() */
	int lines = 1;
%}

DIGIT    [0-9]
ID       [a-z][a-z0-9]*

%%

{DIGIT}+    {
                printf( "Integer: %s (%d)\n", yytext,
                        atoi( yytext ) );
                }

{DIGIT}+"."{DIGIT}*        {
                printf( "Float: %s (%g)\n", yytext,
                        atof( yytext ) );
                }

if|then|begin|end|procedure|function        {
                printf( "Keyword: %s\n", yytext );
                }

{ID}        printf( "Identifier: %s\n", yytext );

"+"|"-"|"*"|"/"   printf( "Operator: %s\n", yytext );

"{"[^}\n]*"}"     /* eat up one-line comments */ printf("Comment: %s\n",yytext);

[ \t]+          /* eat up whitespace */

"\n"          printf("Line: %d\n",++lines);

.           printf( "Unrecognized character: %s\n", yytext );

%%

int main( int argc, char *argv[] )
{
	++argv, --argc;  /* skip over program name */
        if ( argc > 0 )
                yyin = fopen( argv[0], "r" );
        else
                yyin = stdin;
        
	yylex();
	printf("\n%d lines\n",lines);
}